@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2001-2015 Aymeric MOIZARD amoizard@antisip.com
+  Copyright (C) 2001-2020 Aymeric MOIZARD amoizard@antisip.com
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -1388,7 +1388,7 @@ _eXosip_call_retry_request (struct eXosip_t *excontext, eXosip_call_t * jc, eXos
     jd->d_dialog->local_cseq++;
   }
 
-  i = _eXosip_update_top_via (excontext, msg);
+  i = _eXosip_update_top_via (msg);
   if (i != 0) {
     osip_message_free (msg);
     return i;
