@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2001-2015 Aymeric MOIZARD amoizard@antisip.com
+  Copyright (C) 2001-2020 Aymeric MOIZARD amoizard@antisip.com
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -65,6 +65,10 @@ struct eXtl_protocol {
   int (*tl_open) (struct eXosip_t * excontext);
   int (*tl_set_fdset) (struct eXosip_t * excontext, fd_set * osip_fdset, fd_set * osip_wrset, int *fd_max);
   int (*tl_read_message) (struct eXosip_t * excontext, fd_set * osip_fdset, fd_set * osip_wrset);
+
+#ifdef HAVE_SYS_EPOLL_H
+  int (*tl_epoll_read_message) (struct eXosip_t *excontext, int nfds, struct epoll_event* ep_array);
+#endif
   int (*tl_send_message) (struct eXosip_t * excontext, osip_transaction_t * tr, osip_message_t * sip, char *host, int port, int out_socket);
   int (*tl_keepalive) (struct eXosip_t * excontext);
   int (*tl_set_socket) (struct eXosip_t * excontext, int socket);
